@@ -471,7 +471,7 @@ int _mi_insert(register MI_INFO *info, register MI_KEYDEF *keyinfo,
   uchar *endpos, *prev_key;
   MI_KEY_PARAM s_temp;
   DBUG_ENTER("_mi_insert");
-  DBUG_PRINT("enter",("key_pos: 0x%lx", (long) key_pos));
+  DBUG_PRINT("enter",("key_pos: %p", key_pos));
   DBUG_EXECUTE("key",_mi_print_key(DBUG_FILE,keyinfo->seg,key,USE_WHOLE_KEY););
 
   nod_flag=mi_test_if_nod(anc_buff);
@@ -492,8 +492,8 @@ int _mi_insert(register MI_INFO *info, register MI_KEYDEF *keyinfo,
   {
     DBUG_PRINT("test",("t_length: %d  ref_len: %d",
 		       t_length,s_temp.ref_length));
-    DBUG_PRINT("test",("n_ref_len: %d  n_length: %d  key_pos: 0x%lx",
-		       s_temp.n_ref_length,s_temp.n_length, (long) s_temp.key));
+    DBUG_PRINT("test",("n_ref_len: %d  n_length: %d  key_pos: %p",
+		       s_temp.n_ref_length,s_temp.n_length, s_temp.key));
   }
 #endif
   if (t_length > 0)
@@ -543,7 +543,7 @@ int _mi_insert(register MI_INFO *info, register MI_KEYDEF *keyinfo,
       get_key_length(alen,a);
       DBUG_ASSERT(info->ft1_to_ft2==0);
       if (alen == blen &&
-          ha_compare_text(keyinfo->seg->charset, a, alen, b, blen, 0, 0)==0)
+          ha_compare_text(keyinfo->seg->charset, a, alen, b, blen, 0)==0)
       {
         /* yup. converting */
         info->ft1_to_ft2=(DYNAMIC_ARRAY *)
@@ -693,8 +693,8 @@ uchar *_mi_find_half_pos(uint nod_flag, MI_KEYDEF *keyinfo, uchar *page,
   } while (page < end);
   *return_key_length=length;
   *after_key=page;
-  DBUG_PRINT("exit",("returns: 0x%lx  page: 0x%lx  half: 0x%lx",
-                     (long) lastpos, (long) page, (long) end));
+  DBUG_PRINT("exit",("returns: %p  page: %p  half: %p",
+                     lastpos, page, end));
   DBUG_RETURN(lastpos);
 } /* _mi_find_half_pos */
 
@@ -750,8 +750,8 @@ static uchar *_mi_find_last_pos(MI_KEYDEF *keyinfo, uchar *page,
 
   *return_key_length=last_length;
   *after_key=lastpos;
-  DBUG_PRINT("exit",("returns: 0x%lx  page: 0x%lx  end: 0x%lx",
-                     (long) prevpos,(long) page,(long) end));
+  DBUG_PRINT("exit",("returns: %p  page: %p  end: %p",
+                     prevpos, page, end));
   DBUG_RETURN(prevpos);
 } /* _mi_find_last_pos */
 
@@ -929,13 +929,14 @@ static int keys_compare(bulk_insert_param *param, uchar *key1, uchar *key2)
 }
 
 
-static int keys_free(uchar *key, TREE_FREE mode, bulk_insert_param *param)
+static int keys_free(void* key_arg, TREE_FREE mode, void *param_arg)
 {
   /*
     Probably I can use info->lastkey here, but I'm not sure,
     and to be safe I'd better use local lastkey.
   */
-  uchar lastkey[HA_MAX_KEY_BUFF];
+  bulk_insert_param *param= (bulk_insert_param*)param_arg;
+  uchar lastkey[HA_MAX_KEY_BUFF], *key= (uchar*)key_arg;
   uint keylen;
   MI_KEYDEF *keyinfo;
 
@@ -951,14 +952,15 @@ static int keys_free(uchar *key, TREE_FREE mode, bulk_insert_param *param)
     keyinfo=param->info->s->keyinfo+param->keynr;
     keylen=_mi_keylength(keyinfo, key);
     memcpy(lastkey, key, keylen);
-    return _mi_ck_write_btree(param->info,param->keynr,lastkey,
-			      keylen - param->info->s->rec_reflength);
+    _mi_ck_write_btree(param->info, param->keynr, lastkey,
+			                 keylen - param->info->s->rec_reflength);
+    return 0;
   case free_end:
     if (param->info->s->concurrent_insert)
       mysql_rwlock_unlock(&param->info->s->key_root_lock[param->keynr]);
     return 0;
   }
-  return -1;
+  return 0;
 }
 
 
@@ -1014,8 +1016,7 @@ int mi_init_bulk_insert(MI_INFO *info, size_t cache_size, ha_rows rows)
       init_tree(&info->bulk_insert[i],
                 cache_size * key[i].maxlength,
                 cache_size * key[i].maxlength, 0,
-		(qsort_cmp2)keys_compare,
-		(tree_element_free) keys_free, (void *)params++, MYF(0));
+                (qsort_cmp2)keys_compare, keys_free, (void *)params++, MYF(0));
     }
     else
      info->bulk_insert[i].root=0;
@@ -1033,8 +1034,9 @@ void mi_flush_bulk_insert(MI_INFO *info, uint inx)
   }
 }
 
-void mi_end_bulk_insert(MI_INFO *info)
+int mi_end_bulk_insert(MI_INFO *info, my_bool abort)
 {
+  int first_error= 0;
   if (info->bulk_insert)
   {
     uint i;
@@ -1042,10 +1044,16 @@ void mi_end_bulk_insert(MI_INFO *info)
     {
       if (is_tree_inited(& info->bulk_insert[i]))
       {
-        delete_tree(& info->bulk_insert[i]);
+        int error;
+        if ((error= delete_tree(& info->bulk_insert[i], abort)))
+        {
+          first_error= first_error ? first_error : error;
+          abort= 1;
+        }
       }
     }
     my_free(info->bulk_insert);
     info->bulk_insert=0;
   }
+  return first_error;
 }
